/* table-pending-requests */

#table-pending-requests tbody td:nth-child(6) {
    text-align: right;
    white-space: nowrap;
}

/* table-closed-requests */

#table-closed-requests tbody td:nth-child(7) {
    white-space: nowrap;
}
