{% extends 'allianceauth/base.html' %}
{% load i18n %}

{% block page_title %}{{ page_title }} - {% translate "Leave of Absence" %}{% endblock %}

{% block content %}
    <br />
    {% include 'inactivity/partials/menu.html' %}
    <h2>{{ page_title }}</h2>
    {% block details %}{% endblock %}
{% endblock %}
