{% extends 'inactivity/base.html' %}
{% load i18n %}
{% load humanize %}
{% load static %}
{% load bootstrap %}

{% block details %}
    <div class="panel panel-primary">
        <div class="panel-heading">
            <h3 class="panel-title">{% translate "Deny Request" %}</h3>
        </div>

        <div class="panel-body" style="min-height: 100px;">
            <form role="form" method="POST" action="{% url 'inactivity:deny_loa_request' loa_request.pk %}">
                {% csrf_token %}
                {{ form|bootstrap }}
                <div class="form-group">
                    <a class="btn btn-default" href="{% url 'inactivity:manage_requests' %}">{% translate "Cancel" %}</a>
                    <div class="pull-right">
                        <button type="submit" class="btn btn-danger">Deny</button>
                    </div>
                </div>
            </form>
        </div>
    </div>
{% endblock %}
