# Generated by Django 4.0.10 on 2023-06-12 12:06

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("inactivity", "0004_django4_update"),
    ]

    operations = [
        migrations.AddField(
            model_name="inactivityping",
            name="last_login_at",
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="leaveofabsence",
            name="created_at",
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="leaveofabsence",
            name="reason",
            field=models.TextField(
                blank=True,
                default=None,
                help_text="Reason for rejecting a request",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="inactivityping",
            name="timestamp",
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name="inactivityping",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="leaveofabsence",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="Description what this leave of absence request is about.",
                verbose_name="description",
            ),
        ),
        migrations.AlterField(
            model_name="leaveofabsence",
            name="user",
            field=models.ForeignKey(
                help_text="The requestor",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="leave_of_absence_requests",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
